@@ -178,6 +178,25 @@ def token_option(f):
     )(f)
 
 
+def custom_header_option(f):
+    def callback(ctx, _, value):
+        state = ctx.ensure_object(State)
+        value = value or variables.get_variable("custom_header_option")
+        state.custom_header_option = value
+        return value
+
+    return click.option(
+        "-ch",
+        "--custom_header",
+        help="pass one or multiple headers ex.-ch key1:val1",
+        multiple=True,
+        default=0,
+        type=str,
+        expose_value=False,
+        callback=callback,
+    )(f)
+
+
 def host_option(f):
     def callback(ctx, _, value):
         state = ctx.ensure_object(State)
@@ -236,4 +255,5 @@ def common_options(f):
     f = schemes_option(f)
     f = host_option(f)
     f = hard_option(f)
+    f = custom_header_option(f)
     return f
