@@ -3,6 +3,7 @@ from . import (
     annotations,
     contents,
     genes,
+    mega_match,
     therapies,
     transcripts,
     trials,
@@ -15,6 +16,7 @@ __all__ = (
     "annotations",
     "contents",
     "genes",
+    "mega_match",
     "therapies",
     "transcripts",
     "trials",
