@@ -1,4 +1,5 @@
 import re
+import logging
 from functools import partial
 from typing import Iterable
 
@@ -212,7 +213,7 @@ def sync_annotate_sv_match(
                     f"\t(expected valid SVTYPEs {SV_MUTATION_TYPES})\n"
                     "Annotations SV Match Process Terminated"
                 )
-                raise ValueError(msg)
+                logging.log(logging.ERROR, msg)
 
             if record["mutation_type"] == "BND":
                 if record["id"] not in seen_bnd_ids:
