@@ -5,6 +5,7 @@ from .doctypes import (
     __CHILD__,
     is_call_or_variant,
     is_not_skipped_call,
+    filter_out_ref_unknown,
 )
 
 __all__ = (
@@ -15,4 +16,5 @@ __all__ = (
     "is_call_or_variant",
     "is_not_skipped_call",
     "iterate_tuxedo_files",
+    "filter_out_ref_unknown",
 )
