@@ -262,6 +262,248 @@ class AggregatedOmimFileSource(LazyFileSource):
             "file_path": self.name,
         }
 
+@curry
+class AggregatedCOSMICNonSNVSources(LazyFileSource):
+    """
+    This particular file source consumes only a TSV file rather
+    than a combination of a TSV and a VCF like the standard
+    AggregatedCOSMICSource. This is due to the fact that one of these
+    TSV files contain either information about CNV mutations or Fusions
+
+    The logic to process both of these is extremely similar, the column
+    names of each type are just a little different which is why this exists
+    as one class rather than two. Additionally, this file source takes a
+    required argument:
+        record_type = Click.choice(["cnv", "fusion"])
+    and this controls which file we are expecting to receive
+    """
+    def __init__(self, filename, record_type, include_header=True, **meta):
+        self.cosmic_tsv = filename
+        self.include_header = include_header
+        self.meta = meta
+        self.record_type = record_type
+
+        if __TYPE__ not in meta:
+            self.meta = assoc(self.meta, __TYPE__, DocType.AGGREGATE.value)
+        super().__init__(filename)
+
+    def __iter__(self):
+        merged_records = self.aggregate_records()
+        merged_records = self.process_tissue_freqs(merged_records)
+        for _, value in merged_records.items():
+            value["__type__"] = DocType.AGGREGATE.value
+            yield value
+
+    def aggregate_records(self):
+        # noinspection PyUnresolvedReferences
+        if self.record_type == "cnv":
+            columns = COSMIC_TSV_CNV_COLUMNS
+        else:
+            columns = COSMIC_TSV_FUSION_COLUMNS
+
+        file_source = DelimitedFileSource(
+            filename=self.cosmic_tsv,
+            columns=columns,
+            delimiter="\t",
+            skip_comment=True,
+            comment_char="##",
+            include_header=False,
+        )
+
+        cosmic_dict = {}
+        for record in file_source:
+            # Don't iterate on the row with header information
+            if list(record.values())[:-1] != columns:
+                merged_dict = cosmic_dict
+                if self.record_type == "cnv":
+                    self.aggregate_cnv_record(merged_dict, record)
+                elif self.record_type == "fusion":
+                    self.aggregate_fusion_record(merged_dict, record)
+        return cosmic_dict
+
+    def aggregate_cnv_record(self, merged_dict, cnv_row):
+        if "_E" not in cnv_row["gene_name"]:
+            mut_type = "Amplification" if cnv_row['MUT_TYPE'] == "gain" \
+                else cnv_row['MUT_TYPE'].title()
+
+            alteration = f"{cnv_row['gene_name']} {mut_type}"
+            aggregated_cnv_record = merged_dict.setdefault(
+                alteration,
+                {
+                    "CNT": 0,
+                    "TISSUES": {},
+                    "TISSUES_FREQ": {},
+                    "TISSUES_SUBTYPE": {},
+                    "TISSUES_SUBTYPE_FREQ": {},
+                    "HISTOLOGY": {},
+                    "HISTOLOGY_FREQ": {},
+                    "HISTOLOGY_SUBTYPE": {},
+                    "HISTOLOGY_SUBTYPE_FREQ": {},
+                    "ID_SAMPLE": [],
+                    "chr": cnv_row["Chromosome:G_Start..G_Stop"]
+                },
+            )
+
+            aggregated_cnv_record = self.aggregate(
+                aggregated_cnv_record, cnv_row, [alteration], self.record_type
+            )
+            merged_dict[alteration] = aggregated_cnv_record
+
+    def aggregate_fusion_record(self, merged_dict, fusion_row):
+        if "_ENTS" not in fusion_row["3'_GENE_NAME"] and "_ENTS" not in fusion_row["5'_GENE_NAME"]:
+            gene = fusion_row["3'_GENE_NAME"]
+            partner = fusion_row["5'_GENE_NAME"]
+            alteration = (
+                f"{gene}-{partner} Fusion",
+                f"{partner}-{gene} Fusion"
+            )
+
+            if merged_dict.get((alteration[::-1])):
+                alteration = alteration[::-1]
+
+            aggregated_fusion_record = merged_dict.setdefault(
+                alteration,
+                {
+                    "CNT": 0,
+                    "TISSUES": {},
+                    "TISSUES_FREQ": {},
+                    "TISSUES_SUBTYPE": {},
+                    "TISSUES_SUBTYPE_FREQ": {},
+                    "HISTOLOGY": {},
+                    "HISTOLOGY_FREQ": {},
+                    "HISTOLOGY_SUBTYPE": {},
+                    "HISTOLOGY_SUBTYPE_FREQ": {},
+                    "ID_SAMPLE": []
+                },
+            )
+            aggregated_fusion_record = self.aggregate(
+                aggregated_fusion_record, fusion_row, alteration, self.record_type
+            )
+            merged_dict[alteration] = aggregated_fusion_record
+
+
+    def aggregate(self, agg, x, alteration, record_type):  # pragma: no mccabe
+        # COSMIC cnv and fusion .tsv files have different headers for each row
+        # This differentiates between what keys we should be using to accession
+        # values from the tsv row (x)
+        if record_type == "cnv":
+            ID_SAMPLE = "ID_SAMPLE"
+            PRIMARY_SITE = "Primary site"
+            SITE_SUBTYPES = [
+                "Site subtype 1",
+                "Site subtype 2",
+                "Site subtype 3"
+            ]
+            PRIMARY_HISTOLOGY = "Primary histology"
+            HISTOLOGY_SUBTYPES = [
+                "Histology subtype 1",
+                "Histology subtype 2",
+                "Histology subtype 3",
+            ]
+
+        else:
+            ID_SAMPLE = "SAMPLE_NAME"
+            PRIMARY_SITE = "PRIMARY_SITE"
+            SITE_SUBTYPES = [
+                "SITE_SUBTYPE_1",
+                "SITE_SUBTYPE_2",
+                "SITE_SUBTYPE_3"
+            ]
+            PRIMARY_HISTOLOGY = "PRIMARY_HISTOLOGY"
+            HISTOLOGY_SUBTYPES = [
+                "HISTOLOGY_SUBTYPE_1",
+                "HISTOLOGY_SUBTYPE_2",
+                "HISTOLOGY_SUBTYPE_3",
+            ]
+
+        # add the alteration
+        if "alterations" not in agg:
+            agg["alterations"] = alteration
+        else:
+            # throw exception if the gene name for this row
+            # does not match the gene name previously found
+            # for this mutation ID
+            if alteration != agg["alterations"]:
+                raise Exception(
+                    f"TSV data error. Sample ID {x.get(ID_SAMPLE)} "
+                    f"contains more than one value for Gene name. Values "
+                    f"found are: {agg['alterations']} and {alteration}."
+                )
+
+        # Add the gene name to the aggregated dict
+        if x.get("gene_name", None) and "gene_name" not in agg:
+            agg["gene_name"] = [x.get("gene_name")]
+        elif x.get("gene_name"):
+            # throw exception if the gene name for this row
+            # does not match the gene name previously found
+            # for this mutation ID
+            if [x.get("gene_name")] != agg["gene_name"]:
+                raise Exception(
+                    f"TSV data error. Sample ID {x.get(ID_SAMPLE)} "
+                    f"contains more than one value for Gene name. Values "
+                    f"found are: {agg['gene_name']} and {[x.get('gene_name')]}."
+                )
+
+
+        # add the sample ID to the aggregated dict
+        id_sample = x.get(ID_SAMPLE)
+        if id_sample not in agg["ID_SAMPLE"]:
+            agg["ID_SAMPLE"].append(id_sample)
+            # Update the counts only for each unique
+            # alteration + ID sample combo
+            agg["CNT"] = len(agg["ID_SAMPLE"])
+
+            # Puts together aggregates for information regarding site subtypes
+            # Subtypes are structured as <primary_site_value>/<site_subtype_n>
+            for field in SITE_SUBTYPES:
+                if x.get(field) and x.get(field) != "NS":
+                    subtype_string = x.get(PRIMARY_SITE) + "/" + x.get(field)
+                    if subtype_string in agg["TISSUES_SUBTYPE"]:
+                        agg["TISSUES_SUBTYPE"][subtype_string] += 1
+                    else:
+                        agg["TISSUES_SUBTYPE"][subtype_string] = 1
+
+            # Puts together counts for Primary Sites
+            if x.get(PRIMARY_SITE) in agg["TISSUES"]:
+                agg["TISSUES"][x.get(PRIMARY_SITE)] += 1
+            else:
+                agg["TISSUES"][x.get(PRIMARY_SITE)] = 1
+
+            # Puts together aggregates for information regarding site subtypes
+            # Subtypes are structured as <histology_site_value>/<hist_subtype_n>
+            for field in HISTOLOGY_SUBTYPES:
+                if x.get(field) and x.get(field) != "NS":
+                    subtype_string = x.get(PRIMARY_HISTOLOGY) + "/" + x.get(field)
+                    if subtype_string in agg["HISTOLOGY_SUBTYPE"]:
+                        agg["HISTOLOGY_SUBTYPE"][subtype_string] += 1
+                    else:
+                        agg["HISTOLOGY_SUBTYPE"][subtype_string] = 1
+
+            # Puts together counts for Primary Histology
+            if x.get(PRIMARY_HISTOLOGY) in agg["HISTOLOGY"]:
+                agg["HISTOLOGY"][x.get(PRIMARY_HISTOLOGY)] += 1
+            else:
+                agg["HISTOLOGY"][x.get(PRIMARY_HISTOLOGY)] = 1
+
+        return agg
+
+
+    def process_tissue_freqs(self, cosmic_dict):
+        for ck, cv in cosmic_dict.items():
+            for k, v in cv["TISSUES"].items():
+                cosmic_dict[ck]["TISSUES_FREQ"][k] = float(v) / cv["CNT"]
+            for k, v in cv["TISSUES_SUBTYPE"].items():
+                cosmic_dict[ck]["TISSUES_SUBTYPE_FREQ"][k] = (
+                    float(v) / cv["CNT"]
+                )
+            for k, v in cv["HISTOLOGY"].items():
+                cosmic_dict[ck]["HISTOLOGY_FREQ"][k] = float(v) / cv["CNT"]
+            for k, v in cv["HISTOLOGY_SUBTYPE"].items():
+                cosmic_dict[ck]["HISTOLOGY_SUBTYPE_FREQ"][k] = (
+                    float(v) / cv["CNT"]
+                )
+        return cosmic_dict
+
 
 @curry
 class AggregatedCOSMICSources(LazyFileSource):
@@ -597,6 +839,12 @@ class AggregatedCOSMICSources(LazyFileSource):
                     "CNT": 0,
                     "TISSUES": {},
                     "TISSUES_FREQ": {},
+                    "TISSUES_SUBTYPE": {},
+                    "TISSUES_SUBTYPE_FREQ": {},
+                    "HISTOLOGY": {},
+                    "HISTOLOGY_FREQ": {},
+                    "HISTOLOGY_SUBTYPE": {},
+                    "HISTOLOGY_SUBTYPE_FREQ": {},
                     "RESISTANCE_MUTATION": {},
                 },
             )
@@ -702,10 +950,55 @@ class AggregatedCOSMICSources(LazyFileSource):
 
         # update the counts for the tissue sites and resistance mutations
         agg["CNT"] += 1
+        # Puts together aggregates for information regarding site subtypes
+        # Subtypes are structured as <primary_site_value>/<site_subtype_n>
+        site_subtypes = ["Site subtype 1", "Site subtype 2", "Site subtype 3"]
+        for field in site_subtypes:
+            if x.get(field) and x.get(field) != "NS":
+                if (
+                    x.get("Primary site") + "/" + x.get(field)
+                    in agg["TISSUES_SUBTYPE"]
+                ):
+                    agg["TISSUES_SUBTYPE"][
+                        x.get("Primary site") + "/" + x.get(field)
+                    ] += 1
+                else:
+                    agg["TISSUES_SUBTYPE"][
+                        x.get("Primary site") + "/" + x.get(field)
+                    ] = 1
+        # Puts together counts for Primary Sites
         if x.get("Primary site") in agg["TISSUES"]:
             agg["TISSUES"][x.get("Primary site")] += 1
         else:
             agg["TISSUES"][x.get("Primary site")] = 1
+
+        # Puts together aggregates for information regarding site subtypes
+        # Subtypes are structured as <histology_site_value>/<hist_subtype_n>
+        histology_subtypes = [
+            "Histology subtype 1",
+            "Histology subtype 2",
+            "Histology subtype 3",
+        ]
+        for field in histology_subtypes:
+            if x.get(field) and x.get(field) != "NS":
+                if (
+                    x.get("Primary histology") + "/" + x.get(field)
+                    in agg["HISTOLOGY_SUBTYPE"]
+                ):
+                    agg["HISTOLOGY_SUBTYPE"][
+                        x.get("Primary histology") + "/" + x.get(field)
+                    ] += 1
+                else:
+                    agg["HISTOLOGY_SUBTYPE"][
+                        x.get("Primary histology") + "/" + x.get(field)
+                    ] = 1
+
+        # Puts together counts for Primary Histology
+        if x.get("Primary histology") in agg["HISTOLOGY"]:
+            agg["HISTOLOGY"][x.get("Primary histology")] += 1
+        else:
+            agg["HISTOLOGY"][x.get("Primary histology")] = 1
+
         if x.get("Resistance Mutation") in agg["RESISTANCE_MUTATION"]:
             agg["RESISTANCE_MUTATION"][x.get("Resistance Mutation")] += 1
         else:
@@ -716,6 +1009,16 @@ class AggregatedCOSMICSources(LazyFileSource):
         for ck, cv in cosmic_dict.items():
             for k, v in cv["TISSUES"].items():
                 cosmic_dict[ck]["TISSUES_FREQ"][k] = float(v) / cv["CNT"]
+            for k, v in cv["TISSUES_SUBTYPE"].items():
+                cosmic_dict[ck]["TISSUES_SUBTYPE_FREQ"][k] = (
+                    float(v) / cv["CNT"]
+                )
+            for k, v in cv["HISTOLOGY"].items():
+                cosmic_dict[ck]["HISTOLOGY_FREQ"][k] = float(v) / cv["CNT"]
+            for k, v in cv["HISTOLOGY_SUBTYPE"].items():
+                cosmic_dict[ck]["HISTOLOGY_SUBTYPE_FREQ"][k] = (
+                    float(v) / cv["CNT"]
+                )
         return cosmic_dict
 
     def process_vcf_info(self, v):
@@ -793,3 +1096,61 @@ COSMIC_TSV_COLUMNS = [
     "HGVSC",
     "HGVSG",
 ]
+
+COSMIC_TSV_CNV_COLUMNS = [
+    "CNV_ID",
+    "ID_GENE",
+    "gene_name",
+    "ID_SAMPLE",
+    "ID_TUMOUR",
+    "Primary site",
+    "Site subtype 1",
+    "Site subtype 2",
+    "Site subtype 3",
+    "Primary histology",
+    "Histology subtype 1",
+    "Histology subtype 2",
+    "Histology subtype 3",
+    "SAMPLE_NAME",
+    "TOTAL_CN",
+    "MINOR_ALLELE",
+    "MUT_TYPE",
+    "ID_STUDY",
+    "GRCh",
+    "Chromosome:G_Start..G_Stop"
+]
+
+COSMIC_TSV_FUSION_COLUMNS = [
+    "SAMPLE_ID",
+    "SAMPLE_NAME",
+    "PRIMARY_SITE",
+    "SITE_SUBTYPE_1",
+    "SITE_SUBTYPE_2",
+    "SITE_SUBTYPE_3",
+    "PRIMARY_HISTOLOGY",
+    "HISTOLOGY_SUBTYPE_1",
+    "HISTOLOGY_SUBTYPE_2",
+    "HISTOLOGY_SUBTYPE_3",
+    "FUSION_ID",
+    "TRANSLOCATION_NAME",
+    "5'_CHROMOSOME",
+    "5'_STRAND",
+    "5'_GENE_ID",
+    "5'_GENE_NAME",
+    "5'_LAST_OBSERVED_EXON",
+    "5'_GENOME_START_FROM",
+    "5'_GENOME_START_TO",
+    "5'_GENOME_STOP_FROM",
+    "5'_GENOME_STOP_TO",
+    "3'_CHROMOSOME",
+    "3'_STRAND",
+    "3'_GENE_ID",
+    "3'_GENE_NAME",
+    "3'_FIRST_OBSERVED_EXON",
+    "3'_GENOME_START_FROM",
+    "3'_GENOME_START_TO",
+    "3'_GENOME_STOP_FROM",
+    "3'_GENOME_STOP_TO",
+    "FUSION_TYPE",
+    "PUBMED_PMID"
+]
