@@ -1,4 +1,5 @@
 import xml.etree.cElementTree as etree
+import re
 
 from cytoolz import curry
 
@@ -192,12 +193,7 @@ class ClinvarXMLSource(LazyFileSource):
         genes, is_submitted = self.get_genes(interpreted_record)
         if len(genes) > 0 and is_submitted and protein_change:
             gene_name = genes[0]
-            # Catch and remove c. in alt in creation of alteration_string
-            alt_string = f"{gene_name} {protein_change}"
-            if "c." not in alt_string:
-                return alt_string
-            else:
-                return None
+            return f"{gene_name} {protein_change}"
         else:
             return None
 
@@ -260,7 +256,9 @@ class ClinvarXMLSource(LazyFileSource):
         for interpretation in interpretations:
             result = [
                 i.strip()
-                for i in interpretation.find(DESCRIPTION).text.split(",")
+                for i in re.split(
+                    "[,;]", interpretation.find(DESCRIPTION).text
+                )
             ]
         return result
 
@@ -349,9 +347,7 @@ class ClinvarXMLSource(LazyFileSource):
         review_statuses = [interpreted_record.find(REVIEW_STATUS).text]
         clnvi = [self.create_combined_xref(interpreted_record)]
         clndn = self.get_interpreted_conditions(rcv_accessions)
-        gene_protein_change = self.get_alteration_string(interpreted_record)
         csra = self.create_csra(interpreted_record)
-        alteration_string = gene_protein_change
         hgvs_g = csra
         csra_array = csra.split("|") if csra else []
         chromosome = None
@@ -364,7 +360,6 @@ class ClinvarXMLSource(LazyFileSource):
 
         # build top-level result dict with fields that could be parsed
         record = {}
-        alterations = set()
         if clinsig is not None:
             record["CLNSIG__mstring"] = clinsig
         if variation_id is not None:
@@ -375,10 +370,6 @@ class ClinvarXMLSource(LazyFileSource):
             record["CLNVI__mstring"] = clnvi
         if clndn is not None:
             record["CLNDN__mstring"] = clndn
-        # if c. still in alteration_string do not include in record
-        if alteration_string and "c." not in alteration_string:
-            record["alteration__string"] = alteration_string
-            alterations.add(alteration_string)
         if hgvs_g:
             record["hgvs_g"] = hgvs_g
         if chromosome:
@@ -389,7 +380,6 @@ class ClinvarXMLSource(LazyFileSource):
             record["gene"] = genes
         if accession is not None:
             record["accession__string"] = accession
-        record["alterations"] = list(alterations)
         record["csra"] = csra
         record = self.iterate_individual_submissions(
             clinical_assertions, record, interpreted_record
