@@ -177,29 +177,6 @@ NAME_MAPPING = {
 }
 
 
-def add_alteration(ann):
-    alterations = set()
-    genes = ann.get("gene", [])
-    p_dots = ann.get("HGVSp_ANNOVAR__mstring", [])
-    if len(genes) != len(p_dots):
-        error_message = (
-            f"{ann.get('hgvs_g', '')} has {len(genes)} "
-            f"genes and {len(p_dots)} p_dots. These must match."
-        )
-        logger.get_logger().warning(error_message)
-        return []
-    # clean p_dots --> replace X with * and strip out "p."
-    cleaned_p_dots = []
-    for p_dot in p_dots:
-        if p_dot is not None:
-            p_dot = p_dot.split("p.")[-1].replace("X", "*")
-        cleaned_p_dots.append(p_dot)
-    for i in range(0, len(genes)):
-        if genes[i] is not None and cleaned_p_dots[i] is not None:
-            alterations.add(f"{genes[i]} {cleaned_p_dots[i]}")
-    return list(alterations)
-
-
 def clean_genes(ann):
     return ann.get("gene", "").split(";")
 
@@ -210,7 +187,6 @@ register(
     transformer=compose(
         filter_out_empty_chr_start,
         additional_clean_up,
-        lambda x: assoc(x, "alterations", add_alteration(x)),
         lambda x: assoc(x, "csra", kms.annotations.to_csra(x)),
         lambda x: assoc(x, "hgvs_g", kms.annotations.to_csra(x)),
         lambda x: assoc(x, "gene", clean_genes(x)),
