@@ -30,7 +30,7 @@ def parse_values(x):
 
 NAME_MAPPING = {
     "gene": "gene",
-    "gene_id": "ensembl_id",
+    "ensembl_gene_id": "gene_id",
     "pLI_score__float": "pLI",
     "mis_z__float": "mis_z",
     "syn_z__float": "syn_z",
