@@ -10,16 +10,11 @@ NAME_MAPPING = {
 }
 
 
-def clean_hgnc_id(x):
-    return x.strip("HGNC:") if x else None
-
-
 register(
     input_type=DocType.TSV,
     output_type=DocType.HGMD_GENE_LIST,
     transformer=compose(
         lambda x: assoc(x, __TYPE__, DocType.HGMD_GENE_LIST.value),
-        lambda x: assoc(x, "hgnc_id", clean_hgnc_id(x)),
         lambda x: assoc(x, "is_gene_annotation", True),
         lambda x: assoc(x, "is_hgmd_gene__boolean", True),
         name_mapping(NAME_MAPPING),
