@@ -4,6 +4,7 @@ from . import ao_ro_dp
 from . import ad_rd_dp
 from . import brli
 from . import clcbio
+from . import fao_fdp
 from . import pheo
 from . import strelka
 from . import usc
@@ -13,6 +14,7 @@ __all__ = (
     "strelka",
     "pheo",
     "clcbio",
+    "fao_fdp",
     "ao_ro_dp",
     "ad_rd_dp",
     "brli",
